/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2002 Ren Rebe and Valentin Ziegler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   A canvas implementation.
 */

#ifndef GSMP_CANVAS
#define GSMP_CANVAS

// GSML stuff:

#include "include.hh"
#include "typedefs.hh"
#include "Basics.hh" // only for limit_min_max

// stl containers
#include <vector>
#include <deque>

// gtk--
#include <gtk--/drawingarea.h>

#include "plugins-gtk/Pixmap.hh"

namespace GSMP
{
  // because off the cross usage no nested classes are possible here :-(
  typedef u_int64_t canvas_coord;
  typedef _area<canvas_coord> canvas_area;
  typedef canvas_area::point canvas_point;
  
  class GsmpCanvas; // forward
  
  // ***
  
  // derived from SigC::Object to make it possible to connect to SigC signals
  class GsmpCanvasItem : public SigC::Object
  {
  public:
    
    GsmpCanvasItem (GsmpCanvas* i_canvas);
    virtual ~GsmpCanvasItem ();
    
    /* interfaces for the canvas */
    // normal canvas events
    virtual void SetZoom (int n_zoom) {};
    // called from the canvas when we have to render some graphics
    virtual void Render (GsmpPixmap* pixmap, const canvas_area& item_area,
			 const canvas_point& dest) = 0;
    
    // user event handlers - return true if the event got handeled
    virtual bool PointerMotion (const canvas_point& p) {return false;};
    virtual bool ButtonPress (int button) {return false;};
    virtual bool BottonRelease (int button) {return false;};
    virtual bool KeyPress (int key) {return false;};
    virtual bool KeyRelease (int key) {return false;};
    
 protected:
    friend class GsmpCanvas;
    bool m_transparent;
    signed int m_layer;
    canvas_area m_area;
    
    GsmpCanvas* m_canvas;
  };
  
  class GsmpCanvas : public Gtk::DrawingArea
  {
  public:
    
    GsmpCanvas ();
    ~GsmpCanvas ();

    /* methods for the outer world */
    
    const canvas_area& Viewport () const;
    void SetViewport (const canvas_point& p);
    
    const canvas_area& CanvasArea () const;
    
    // !the zoom is not performed by some dirty magic - but must be done by the items, itself!
    void SetZoom (int n_zoom);
    int Zoom () const;
    
    void GuiRefresh ();
    
    /* methods for the items */
    bool Add (GsmpCanvasItem* item);
    bool Remove (GsmpCanvasItem* item);
    void RenderRequest (const canvas_area& a);
    
    /* not needed?: void ItemAreaMoved (); // TODO: ??? */
    void ItemAreaEnlarged (const canvas_area& item_area);
    void ItemAreaChanged (const canvas_area& item_area, const canvas_area& old_item_area);
    
    SigC::Signal1<void, int> SignalZoomChanged;
    SigC::Signal1<void, const canvas_area&> SignalAreaResized;
    SigC::Signal1<void, const canvas_area&> SignalViewportModified;
    
    // user events - only signalled if no item processed the event ...
    SigC::Signal1 <void, const canvas_point&> SignalMotionNotify;
    SigC::Signal1 <void, int, const canvas_point&> SignalButtonPress;
    SigC::Signal1 <void, int> SignalButtonRelease;
    SigC::Signal1 <void, int> SignalKeyPress;
    SigC::Signal1 <void, int> SignalKeyRelease;
    
  protected:
    
    void Render ();
    
    // gtk-- event-impl overwrites ...
    virtual gint configure_event_impl (GdkEventConfigure* event);
    virtual gint expose_event_impl (GdkEventExpose* event);
    
    virtual gint motion_notify_event_impl (GdkEventMotion* event);
    virtual gint button_press_event_impl (GdkEventButton* event);
    virtual gint button_release_event_impl (GdkEventButton* event);
    virtual gint key_press_event_impl(GdkEventKey* event);
    virtual gint key_release_event_impl(GdkEventKey* event);
    
    int m_zoom;
    
    canvas_area m_area;
    canvas_area m_viewport;
    
    GsmpColor::Color m_c;
    canvas_point m_pointer;
    
    // gdk-- objects
    Gdk_Window m_win;
    Gdk_GC m_gc;  
    
    // The Gsmp pixmap all anti-aliased and alpha blending can be done ;-) !
    GsmpPixmap m_pixmap;
    
    // Gsmp anti-aliased fonts (based on FreeType2)
    GsmpFontSystem m_fonts;
    GsmpFace* m_face;
    
    // queued render request
    deque <canvas_area> m_render_queue;
    
    // all items
    deque <GsmpCanvasItem*> m_items;
  };

} // end namespace GSMP

#endif
